import axios from "@/api/axios";
import Constant from "@/utils/constant";
import {getToken, getRefreshToken} from "@/utils/tokenUtil";

// 登录方法(oauth)
export function oauthLogin(code) {
  return axios({
    url: "/common/oauth/login",
    method: "post",
    data: {
      code: code,
      equipment: 'pc'
    }
  });
}


// 登录方法
export function login(loginName, password, code, uuid) {
  return axios({
    url: "/login",
    method: "post",
    data: {
      loginName,
      password,
      code,
      uuid
    }
  });
}

// 退出方法
export function logout() {
  return axios({
    url: "/logout",
    method: "post"
  });
}

// 获取验证码
export function getCaptchaImage(loginName) {
  return axios({
    url: "/common/pcCaptcha",
    method: "get",
    params: {loginName: loginName}
  });
}

// 刷新token
export function refreshToken() {
  const headers = {};
  headers[Constant.requestTokenKey] = getToken();
  headers[Constant.requestRefreshTokenKey] = getRefreshToken();
  return axios({
    url: "/common/refreshToken",
    method: "get",
    headers: headers
  });
}

// 加载菜单资源
export function getResource() {
  return axios({
    url: "/core/base/resources/router",
    method: "get"
  });
}

// 获取登录用户
export function getLoginUser() {
  return axios({
    url: "/core/base/user/login-user",
    method: "get"
  });
}

// 表单校验字段
export function validateField(url, params, callback, headers) {
  axios({
    url: url,
    method: "get",
    headers: headers,
    data: params
  }).then(res => {
    // result返回true或者false
    if (res.data.result) {
      callback()
    } else {
      callback(new Error(res.data.info))
    }
  });
}

// 加载资源
export function getList(params, headers) {
  return axios({
    url: "/getList",
    method: "get",
    headers: headers,
    data: params
  });
}

// 加载资源
export function getTree(params, headers) {
  return axios({
    url: "/getTree",
    method: "get",
    headers: headers,
    data: params
  });
}


// 加载资源
export function getTreeDemo(params, headers) {
  return axios({
    url: "/getTreeDemo",
    method: "get",
    headers: headers,
    data: params
  });
}
