import axios from "@/api/axios";

// 根据id获取字典分类信息
export function getCategoryById(id, headers) {
  return axios({
    url: "/core/system/dict-categorys/" + id,
    method: "get",
    headers: headers
  });
}

// 获取字典分类树节点
export function getTreeNodes(root, disableId, headers) {
  return axios({
    url: "/core/system/dict-categorys/tree-nodes",
    method: "get",
    headers: headers,
    data: {root: root, disableId: disableId}
  });
}

// 新增字典分类
export function insertCategory(params, headers) {
  return axios({
    url: "/core/system/dict-categorys",
    method: "post",
    headers: headers,
    data: params
  });
}

// 修改字典分类
export function updateCategory(params, headers) {
  return axios({
    url: "/core/system/dict-categorys",
    method: "put",
    headers: headers,
    data: params
  });
}

// 删除单个字典分类
export function deleteCategorySingle(id, headers) {
  return axios({
    url: "/core/system/dict-categorys/" + id,
    method: "delete",
    headers: headers
  });
}

// 批量删除字典分类
export function deleteCategoryBatch(idList, headers) {
  return axios({
    url: "/core/system/dict-categorys/deletion",
    method: "post",
    isList: true,
    headers: headers,
    data: {idList: idList}
  });
}

// 根据id获取字典信息
export function getDictById(id, headers) {
  return axios({
    url: "/core/system/dicts/" + id,
    method: "get",
    headers: headers
  });
}

// 获取字典分页列表
export function getDictList(params, headers) {
  return axios({
    url: "/core/system/dicts",
    method: "get",
    headers: headers,
    data: params
  });
}

// 新增或修改字典
export function insertOrUpdateDict(params, headers) {
  return axios({
    url: "/core/system/dicts",
    method: "post",
    headers: headers,
    data: params
  });
}

// 批量删除字典
export function deleteDictSingle(id, headers) {
  return axios({
    url: "/core/system/dicts/" + id,
    method: "delete",
    headers: headers
  });
}

// 批量删除字典
export function deleteDictBatch(idList, headers) {
  return axios({
    url: "/core/system/dicts/deletion",
    method: "post",
    isList: true,
    headers: headers,
    data: {idList: idList}
  });
}

// 查询字典选型
export function getOptionsByCode(code) {
  return axios({
    url: "/core/system/dicts/"+code+"/options",
    method: "get"
  });
}

