/**
 * 可编辑表格组件，继承Item
 */

import item from "../Item";
import FileUpload from '../common/FileUpload'

export default {
  name: 'MFileUpload',
  mixins: [item],
  components: {
    FileUpload
  },
  props: {
    fileIdList: {
      type: Array,
      required: true
    },
    moduleCode: {
      type: String,
      default: 'common'
    },
    // 接受的文件类型，形如['.png', '.jpg', '.jpeg', '.gif', '.bmp'...]
    accept: {
      type: Array,
      default: () => []
    },
    // 如果设为true，则代表一次只能选择一个文件。
    single: {
      type: Boolean,
      default: true
    }
  },
  computed: {
    inputSize() {
      if (this.size) return this.size
      if (this.mForm) {
        return this.mForm.$attrs.size || ''
      }
      return ''
    }
  },
  methods: {
    // 初始化文件id集合
    initFileIdList(fileIds) {
      this.$refs.comp.initFileIdList(fileIds)
    }
  },
  render() {
    const vnode = (
      <FileUpload ref="comp"
                  {...{
                    props: {
                      ...this.innerProps, size: this.inputSize, fileIdList: this.fileIdList,
                      moduleCode: this.moduleCode, accept: this.accept, single: this.single
                    },
                    on: {
                      ...this.$listeners
                    }
                  }}
                  size={this.inputSize}
                  vModel={this.fieldValue}>
      </FileUpload>
    )
    return this.renderComponent(vnode)
  }
}
