/**
 * 可编辑表格组件，继承Item
 */

import item from "../Item";
import ImageUpload from '../common/ImageUpload'

export default {
  name: 'MImageUpload',
  mixins: [item],
  components: {
    ImageUpload
  },
  props: {
    fileIdList: {
      type: Array,
      required: true
    },
    moduleCode: {
      type: String,
      default: 'common'
    },
    // 接受的文件类型，形如'.gif, .jpg, .jpeg, .png, .bmp, .webp'
    accept: {
      type: String,
      default: '.gif, .jpg, .jpeg, .png, .bmp, .webp'
    },
    // 是否支持多选
    multiple: {
      type: Boolean,
      default: false
    },
    // 最大允许上传个数
    limit: {
      type: Number,
      default: 30
    }
  },
  computed: {
    inputSize() {
      if (this.size) return this.size
      if (this.mForm) {
        return this.mForm.$attrs.size || ''
      }
      return ''
    }
  },
  methods: {
    // 初始化文件id集合
    initFileIdList(fileIds) {
      this.$refs.comp.initFileIdList(fileIds)
    }
  },
  render() {
    const vnode = (
      <ImageUpload ref="comp"
                   {...{
                     props: {
                       ...this.innerProps, size: this.inputSize, fileIdList: this.fileIdList,
                       moduleCode: this.moduleCode, accept: this.accept, multiple: this.multiple, limit: this.limit
                     },
                     on: {
                       ...this.$listeners
                     }
                   }}
                   size={this.inputSize}
                   vModel={this.fieldValue}>
      </ImageUpload>
    )
    return this.renderComponent(vnode)
  }
}
