/**
 * 标签输入框组件，继承Item
 */

import item from "../Item";
import TagSelect from '../common/TagSelect'

/**
 * 插槽
 * @member slots
 * @property {string} default 作用域插槽，定义每个tag显示内容，参数: item（数据项）,index(索引)
 */
export default {
  name: 'MTagSelect',
  mixins: [item],
  components: {
    TagSelect
  },
  props: {
    // 加载中提示
    loadingText: {
      type: String,
      default: '正在加载选项数据...'
    },
    value: [Array, String, Number]
  },
  computed: {
    inputSize() {
      if (this.size) return this.size
      if (this.mForm) {
        return this.mForm.$attrs.size || ''
      }
      return ''
    }
  },
  methods: {
    getDefaultValue() {
      return this.$attrs.multiple ? [] : ''
    },
    handleChange(val) {
      if (val && val.length) {
        this.clearValidate()
      }
      this.$emit('change', val)
    }
  },
  render() {
    
    const scopedSlots = this.$scopedSlots.default
      ? {
        default: (props) => {
          return this.$scopedSlots.default(props)
        }
        
      }
      : null;
    
    const vnode = this.loading
    ? (
      <div class="m-form__loading">
        {this.$slots.loading || this.loadingText}
      </div>
    ):(
      <TagSelect ref="comp"
                 {...{
                   props: {...this.innerProps, size: this.inputSize},
                   on: {
                     ...this.$listeners,
                     change: this.handleChange
                   },
                   scopedSlots: scopedSlots
                 }}
                 size={this.inputSize}
                 options={this.currentOptions}
                 keyMap={this.keyMap}
                 vModel={this.fieldValue}>
      </TagSelect>
    )
    
    return this.renderComponent(vnode)
  }
}
