/**
 * 标签输入框组件，继承Item
 */

import item from "../Item";
import TreeSelect from '../common/TreeSelect'
import { listToTree } from '@/utils/treeUtil'

export default {
  name: 'MTreeSelect',
  mixins: [item],
  components: {
    TreeSelect
  },
  /**
   * 扩展参数
   * @member props
   * @property {*} [root] 数据项根节点的值
   */
  props: {
    value: [Array, String, Number],
    // 树的根节点，parentId的值
    root: {
      type: [String, Number, Boolean],
      default: null
    }
  },
  data() {
    return {
      optionsTree: [],
    }
  },
  computed: {
    inputSize() {
      if (this.size) return this.size
      if (this.mForm) {
        return this.mForm.$attrs.size || ''
      }
      return ''
    }
  },
  watch: {
    currentOptions: {
      immediate: true,
      handler(val) {
        const { id, parentId } = this.keyMap
        this.optionsTree = listToTree(val || [], this.root, id, parentId)
      }
    },
  },
  methods: {
    getDefaultValue() {
      return this.$attrs.multiple ? [] : ''
    },
    //返回所有选中的节点
    getCheckedNodes() {
      return this.$refs["mItem"].$children[1].checked;
    },
  },
  render() {
    const scopedSlots = this.$scopedSlots.default
      ? {
        default: (props) => {
          return this.$scopedSlots.default(props)
        }

      }
      : null;

    const vnode = (
      <TreeSelect ref="comp"
        {...{
          props: { ...this.innerProps, size: this.inputSize },
          on: {
            ...this.$listeners
          },
          scopedSlots: scopedSlots
        }}
        size={this.inputSize}
        options={this.optionsTree}
        keyMap={this.keyMap}
        vModel={this.fieldValue}>
      </TreeSelect>
    )

    return this.renderComponent(vnode)
  }
}
