import props from "./props.js";
import {listToTree} from '@/utils/treeUtil'
export default {
  name: "MTable",
  provide() {
    return {
      mTable: this
    };
  },
  props,
  data() {
    const _this = this;
    return {
      timer:null,
      //表格数据列表
      tableData: [],
      filterKey: "",
      //表格加载loading动画
      loading: false,
      total: 0,
      pageSize: 20,
      pageSizesList: [10, 20, 30, 40, 50],
      pagination: {
        pageIndex: 1,
        pageSize: 20
      },
    };
  },
  computed: {
    currentToolStyle() {
      if(this.$slots.query){
        return {
          borderTop: "1px dashed #ccc",
          ...this.toolStyle
        }
      }
      return this.toolStyle;
    }
  },
  created() {
    this.filterKey = "";
    if (this.api) {
      this.fetchHandle();
    }
  },
  methods: {
    //获取数据
    fetchHandle(params = {}) {
      this.loading = true;
      let {
        listField,
        totalField,
        pagination,
        headers,
        searchParams,
        rowKey,
      } = this;
      //此处根据4.5框架传参
      params = Object.assign(
        searchParams,
        {
          pageNum: pagination.pageIndex,
          pageSize: pagination.pageSize
        },
        params
      );
      if (this.filterKey) {
        const _fk = this.filterKey.split("-")
        params["sort"] = _fk[0];
        params["order"] = _fk[1];
      }else if(this.defaultSort){
        params["sort"] = this.defaultSort.sort;
        params["order"] = this.defaultSort.order;
      }
      this.api(params, headers).then(response => {
        let result = response.data;
        if (result) {
          if (listField && listField.includes(".")) {
            listField.split(".").forEach(item => {
              result = result[item];
            });
          } else {
            result = result[listField];
          }
        }
        if (!result && !(result instanceof Array)) {
          try {
            throw new Error(`The result of key:${listField} is not Array`);
          } catch (e) {}
          this.loading = false;
          return false;
        }
        if(rowKey){
          this.tableData =  listToTree(result || [],this.root)
        }else{
          this.tableData = result;
        }
        //获取数据总数
        let totalValue = response.data;
        if (totalField) {
          if(totalField.includes(".")){
            totalField.split(".").forEach(item => {
              totalValue = totalValue[item];
            });
          }else {
            totalValue = response.data[totalField];
            if (typeof totalValue === "string" || totalValue instanceof String) {
              totalValue = Number(totalValue)
            }
          }
        }else{
          totalValue= result.length
        }
        this.loading = false;
        this.total = totalValue;
      });
    },
    //处理分页事件
    handleCurrentChange(pageIndex) {
      this.pagination.pageIndex = pageIndex;
      this.fetchHandle();
    },
    handleSizeChange(pageSize) {
      this.pagination.pageSize = pageSize;
      this.fetchHandle();
    },
    //刷新列表
    reloadTable(params) {
      //做查询就从第一页开始
      this.pagination.pageIndex=1
      this.fetchHandle(params);
    },
    emitEventHandler(event) {
      this.$emit(event, ...Array.from(arguments).slice(1));
    },
    //sortable设置为custom，监听排序事件。表格的排序条件发生变化的时候会触发该事件
    handlerSortChange(event) {
      this.filterKey = "";
      if (event.order) {
        this.filterKey = event.prop + "-" + event.order.replace("ending", "");
      }
      //重新查询远程服务器数据
      this.fetchHandle();
    }
  }
};
