import Vue from "vue";
import App from "./App";
import i18n from "./i18n";
import store from "./store";
import router from "./router";
import directive from "./directive"; // 自定义directive
import VueMeta from "vue-meta"; // 头部标签组件
import "@/assets/styles/theme/index.css";
import ElementUI from "element-ui";
import "font-awesome/css/font-awesome.min.css";
import "@/assets/styles/index.scss"; // 全局 css
import "@/assets/styles/element-variables.scss";
import mediway from "@/mediway"; // 全局工具及api
import uploader from 'vue-simple-uploader';

import MIcon from "@/components/MIcon";
import MTable from "@/components/MTable";
import MQuery from "@/components/MQuery";
import MLayout from "@/components/MLayout";
import MButton from "@/components/MButton";
import MTree from "@/components/MTree";
import 'vue-event-calendar/dist/style.css'
import vueEventCalendar from 'vue-event-calendar';
import VueDragResize from 'vue-drag-resize'


// 注册全局组件
import "@/components/MForm"; //自动注册

if (window.__POWERED_BY_QIANKUN__) {
  Vue.component('MButton', window.commonComponent.MButton);
}
if (!window.__POWERED_BY_QIANKUN__) {
  /*Vue.component(MIcon.name, MIcon);
  Vue.component(MTable.name, MTable);
  Vue.component(MQuery.name, MQuery);
  Vue.component(MLayout.name, MLayout);*/
  Vue.component(MButton.name, MButton);
 /* Vue.component(MTree.name, MTree);*/
}

Vue.component(MIcon.name, MIcon);
Vue.component(MTable.name, MTable);
Vue.component(MQuery.name, MQuery);
Vue.component(MLayout.name, MLayout);
//Vue.component('MButton', window.commonComponent.MButton);
Vue.component(MTree.name, MTree);

Vue.use(vueEventCalendar, {locale: 'zh', color: '#fff'});
Vue.component('vue-drag-resize', VueDragResize);

if (process.env.VUE_APP_MOCK) {
  // 引入mockjs
  require("./mock");
}

console.log('mainjs');
console.log(window.commonComponent);

Vue.use(uploader);
Vue.use(directive);
Vue.use(VueMeta);
Vue.use(ElementUI, { size: 'mini', zIndex: 3000 });
Vue.use(mediway); // 挂载全局工具

let instence = null;

function render(props){
	instence = new Vue({
	  el: "#app",
  store,
  router,
  i18n,
  render: h => h(App)
	}).$mount('#app') //这里是挂在到自己的HTML中，基座会拿到挂在后的HTML，将其插入
}
if (window.__POWERED_BY_QIANKUN__) {
	__webpack_public_path__ = window.__INJECTED_PUBLIC_PATH_BY_QIANKUN__
}
if (!window.__POWERED_BY_QIANKUN__) {
  render()
}

export async function bootstrap(props){};
export async function mount(props){
     render(props);
};
export async function unmount(props){
	instence.$destroy();
};
