import Vue from "vue";
import Router from "vue-router";
import NProgress from "nprogress";
import "nprogress/nprogress.css";
import store from "@/store";
import {getToken} from "@/utils/tokenUtil";
import {getResource, getLoginUser} from "@/api/modules/base";
import {listToTree} from "@/utils/treeUtil";
import {md5} from "@/utils/cryptoUtil";
import {getLocalhostPath} from "@/utils/commonUtil";
import Layout from "@/layout";
import ParentView from "@/layout/common/components/ParentView";
import Constant from "@/utils/constant";
import * as CryptoUtil from '@/utils/cryptoUtil'

NProgress.configure({showSpinner: false});

Vue.use(Router);

/* Main */

/**
 * Note: 路由配置项
 *
 * hidden: true                   // 当设置 true 的时候该路由不会再侧边栏出现 如401，login等页面，或者如一些编辑页面/edit/1
 * alwaysShow: true               // 当你一个路由下面的 children 声明的路由大于1个时，自动会变成嵌套的模式--如组件页面
 *                                // 只有一个时，会将那个子路由当做根路由显示在侧边栏--如引导页面
 *                                // 若你想不管路由下面的 children 声明的个数都显示你的根路由
 *                                // 你可以设置 alwaysShow: true，这样它就会忽略之前定义的规则，一直显示根路由
 * redirect: noRedirect           // 当设置 noRedirect 的时候该路由在面包屑导航中不可被点击
 * name:'router-name'             // 设定路由的名字，一定要填写不然使用<keep-alive>时会出现各种问题
 * meta : {
    noCache: true                // 如果设置为true，则不会被 <keep-alive> 缓存(默认 false)
    title: 'title'               // 设置该路由在侧边栏和面包屑中展示的名字
    icon: 'svg-name'             // 设置该路由的图标，对应路径src/assets/icons/svg
    breadcrumb: false            // 如果设置为false，则不会在breadcrumb面包屑中显示
  }
 */

// 公共路由
export const constantRoutes = [
  {
    path: "/redirect",
    component: Layout,
    hidden: true,
    children: [
      {
        path: "/redirect/:path(.*)",
        component: resolve =>
          require(["@/layout/common/components/redirect"], resolve)
      }
    ]
  },
  {
    path: "/login",
    component: resolve => require(["@/views/login/form/index"], resolve),
    hidden: true
  },
  {
    path: "/oauth/callback",
    component: resolve => require(["@/views/login/oauth/callback"], resolve),
    hidden: true
  },
  {
    path: "/404",
    component: resolve => require(["@/views/error/404"], resolve),
    hidden: true
  },
  {
    path: "/401",
    component: resolve => require(["@/views/error/401"], resolve),
    hidden: true
  },
  {
    path: "",
    component: Layout,
    redirect: "/home",
    children: [
      {
        path: "/home",
        //component: resolve => require(["@/views/home"], resolve),
        name: "home",
        iframeComponent:resolve => require(["@/views/home"], resolve),
        props:{
          src:"/home"
        },
        meta: {title: "首页", icon: "dashboard", noCache: true, affix: true}
      }
    ]
  },
  // 动态添加路由的目录
  {
    path: "dynamic_route",
    component: Layout,
    name: "dynamic_route",
    hidden: true,
    children: [{
      path: "/user/profile",
      component: resolve => require(["@/views/system/user/profile"], resolve),
      name: "个人中心",
      meta: {title: "个人中心", icon: "dashboard", noCache: true, affix: false}
    }]
  }
];

const router = new Router({
  mode: "history", // 去掉url中的#
  scrollBehavior: () => ({y: 0}),
  base: '/vue',
  routes: constantRoutes
});

//不显示环形进度动画
NProgress.configure({showSpinner: false});

// 白名单（不需要token就可以访问的前端页面路由）
const whiteList = ["/login", "/oauth/callback"];

//路由前置守卫 - 用于用户认证过滤,加载动态路由
router.beforeEach((to, from, next) => {
  NProgress.start();
  if (getToken()) {
    to.meta.title && store.dispatch("app/setTitle", to.meta.title);
    /* has token*/
    if (to.path === "/login" || to.path === "/oauth/callback") {
      next({path: "/",query:{"menu":to.query.menu}});
      store.dispatch("initMenuType", to.query.menu);
      NProgress.done();
    } else {
      if (!store.getters.loginUser || !store.getters.loginUser.userCode) {
        getLoginUser().then(res => {
          store.dispatch("initLoginUser", res.data.user);
        })
      }
      if (store.getters.dynamicRoutes.length === 0) {
        getResource().then(res => {
          const menuData = JSON.parse(JSON.stringify(res.data.data));
          const _menuData = listToTree(menuData);
          let menuList = [];
          const ifmRoutes = [];//内部页面独立打开，需要独立的路由
          //判断过滤菜单
          if (to.query.menu != undefined && to.query.menu != "") {
            store.dispatch("initMenuType", to.query.menu);
            let new_menu = [];
            for (const item of _menuData) {
              if (to.query.menu == "1") {
                if (item.router == "system" || item.router == "ucm") {
                  new_menu.push(item);
                }
              } else if (to.query.menu == "2") {
                if (item.router == "account" || item.router == "rpm" || item.router ==
                  "apply_permission" /* ||item.router =="pam"||item.router=="pro" */ ) {
                  new_menu.push(item);
                }
              } else if (to.query.menu == "3") {
                if (item.router == "/system" || item.router == "monitor") {
                  new_menu.push(item);
                }
              } else if (to.query.menu == "4") {
                if (item.router == "pam" || item.router == "pro") {
                  new_menu.push(item);
                }
              }
            }
            generaMenu(menuList, new_menu, ifmRoutes);
          } else {
            generaMenu(menuList, _menuData, ifmRoutes);
          }
          console.log(JSON.stringify(menuList));
          // 合并独立页面路由
          menuList = menuList.concat(ifmRoutes)
          // 最后再加入404
          menuList.push({
            path: "*",
            component: loadView("/error/404"),
            hidden: true
          });
          store.dispatch("initRoutes", menuList).then(menus => {
            addAllRoutes(menus)
            delaRefreshRouter(menus.concat(constantRoutes), to)
            next({...to, replace: true}); // hack方法 确保addRoutes已完成
          });
        });
      } else {
        next();
      }
    }
  } else {
    // 没有token
    if (whiteList.indexOf(to.path) !== -1) {
      // 在免登录白名单，直接进入
      next();
    } else {
      if (Constant.loginType === 'form') {
        const url = encodeURIComponent(to.fullPath)
        next(`/login?redirect=${url}`); // 否则全部重定向到登录页
      } else if (Constant.loginType === 'oauth') {
        window.location.href = Constant.oauthServerUrl + '/oauth2.0/authorize?client_id=' + Constant.oauthClientId +
          '&redirect_uri=' + Constant.oauthClientUrl + '&response_type=code&state=' + encodeURIComponent(to.fullPath.replace('?state=', ''))
      }
      NProgress.done();
    }
  }
});

router.afterEach(() => {
  NProgress.done();
});

/**
 * 添加所有路由
 */
function addAllRoutes(menuList) {
  // 非一级菜单路由
  let otherRoutes = [];
  for (let i = 0; i < menuList.length; i++) {
    if (menuList[i].menu) {
      // 如果一级是菜单，动态添加路由
      router.addRoute("dynamic_route", menuList[i]);
    } else {
      otherRoutes.push(menuList[i])
    }
  }
  // 动态添加可访问路由表
  router.addRoutes(otherRoutes);
}

/**
 * 处理刷新时路由管理
 * @param menuData
 * @param to
 */
function delaRefreshRouter(menuData, to) {
  let flag = {release: false}
  dealRelease(flag, menuData, to.path)

  if (!flag.release) {
    // 不放行则说明路由中没有此路由，那么需要加上
    if (to.path.indexOf('/iframe/') !== -1) {
      let compPath = to.path.split('/iframe/')[0]
      const vId = CryptoUtil.md5(to.query.title + ":" + to.query.url)
      if (compPath.startsWith('/single')) {
        router.addRoutes([{
          path: to.path,
          component: resolve => require([`@/layout/common/components/iFrame`], resolve),
          name: vId,
          hidden: true,
          meta: {
            title: to.query.title,
          },
          props: {
            src: to.query.url
          }
        }]);
      } else {
        router.addRoute("dynamic_route", {
          path: to.path,
          component: resolve => require([`@/layout/common/components/iFrame`], resolve),
          name: vId,
          hidden: true,
          meta: {
            title: to.query.title,
          },
          props: {
            src: to.query.url
          }
        });
      }
    } else {
      let compPath = to.path.split('/split')[0]
      if (compPath.startsWith('/single')) {
        compPath = compPath.replace('/single', '')
        router.addRoutes([{
          path: to.path,
          component: loadView(compPath),
          name: compPath.substr(1).replace(new RegExp("/", "gm"), "_"),
          hidden: true,
          meta: {
            title: to.query.title,
          }
        }]);
      } else {
        router.addRoute("dynamic_route", {
          path: to.path,
          component: loadView(compPath),
          name: compPath.substr(1).replace(new RegExp("/", "gm"), "_"),
          hidden: true,
          meta: {
            title: to.query.title,
          }
        });
      }
    }
  }
}

/**
 * 处理是否放行
 * @param flag 放行标识
 * @param menuData 菜单数据
 * @param path 匹配路径
 */
function dealRelease(flag, menuData, path) {
  for (let i = 0; i < menuData.length; i++) {
    if (menuData[i].path === path) {
      flag.release = true
      break
    }
    if (menuData[i].children && menuData[i].children.length > 0) {
      dealRelease(flag, menuData[i].children, path)
    }
  }
}

/**
 * 后台查询的菜单数据拼装成路由格式的数据
 * @param {*} routes
 * @param {*} data
 * @param {*} ifmRoutes
 */
function generaMenu(routes, data, ifmRoutes, noTop) {
  for (const item of data) {
    let menu = {
      hidden: item.hide,
      children: [],
      menu: item.resType === 'menu',
      meta: {
        title: item.name,
        icon: item.icon,
        noCache: item.keepAlive == false
      }
    };
    let t_component = Layout;
    let t_path = item.router;
    //处理路由参数
    if (t_path.indexOf("?") != -1) {
      //包含参数
      t_path = t_path.substring(0, t_path.lastIndexOf("?"));
    }
    let t_name = t_path ? t_path.substr(1).replace(new RegExp("/", "gm"), "_") : "";
    // 如果是目录
    if (item.resType == "catalog") {
      t_component = noTop ? ParentView : Layout;
      t_path = "/top/" + md5(item.name);

    } else {
      // 如果是外链
      if (item.linked) {
        if (item.openType == "iframe") {
          // 如果是iframe固定采用iFrame组件，路由采用地址md5加密保证唯一
          t_path = "/iframe/" + md5(item.router);
          t_component = resolve => require([`@/layout/common/components/iFrame`], resolve);
          // 给iFrame组件传参
          menu["props"] = {src: item.router};
        }
      } else {
        t_path = item.router;
        if (t_path.indexOf("?") != -1) {
          //包含参数
          let objStr = t_path.substring(t_path.lastIndexOf("?") + 1);
          let paramArray = objStr.split("&");
          let query = {};
          for (let i = 0; i < paramArray.length; i++) {
            let oneArray = paramArray[i].split("=");
            query[oneArray[0]] = oneArray[1];
          }
          query['title'] = item.name
          menu["query"] = query;//存储路由参数
          t_path = t_path.substring(0, t_path.lastIndexOf("?"));
          const vId = CryptoUtil.md5(item.name + ":" + t_path)
          t_path = t_path + "/split/" + vId
          t_name = t_name + vId
        }
        //如果是内部打开
        if (item.openType == "inner") {
          t_component = loadView(item.router);
        } else {
          // 真实路由
          const _src = getLocalhostPath() + "/single" + item.router;
          //如果是用iframe打开内部页面
          if (item.openType == "iframe") {
            t_component = resolve => require([`@/layout/common/components/iFrame`], resolve);
            // 给iFrame组件传参
            menu["props"] = {src: _src};
            //如果用浏览器新面签打开内部页面
          } else if (item.openType == "blank") {
            t_path = _src;
          }
        }
        // 如果是内部页面独立打开，需要独立的路由，路由地址加入/single前缀
        let single = {
          path: "/single" + item.router,
          component: loadView(item.router),
          name: md5(item.router),//没啥意义
          hidden: true,
          meta: {
            title: item.name,
            icon: item.icon
          }
        };
        ifmRoutes.push(single)
      }
    }
    menu["path"] = t_path;
    // 如果打开方式是iframe并且需要缓存
    if (item.keepAlive && item.openType === 'iframe') {
      menu["iframeComponent"] = t_component;
    } else {
      menu["component"] = t_component;
    }
    menu["name"] = t_name;
    if (item.children) {
      generaMenu(menu.children, item.children, ifmRoutes, true);
    }
    routes.push(menu);
  }
}

const loadView = path => {
  // 路由懒加载
  return resolve => require([`@/views${path}`], resolve);
};

export default router;
