import * as baseApi from "@/api/modules/base";
import * as tokenUtil from "@/utils/tokenUtil";
import Constant from "@/utils/constant";

// 登录方法(oauth)
export function oauthLogin(code) {
  return new Promise((resolve, reject) => {
    baseApi.oauthLogin(code).then(res => {
      tokenUtil.setAllToken(res.headers[Constant.requestTokenKey], res.headers[Constant.requestRefreshTokenKey]);
      resolve();
    }).catch(error => {
      reject(error);
    });
  });
}

// 登录方法
export function login(loginName, password, code, uuid) {
  return new Promise((resolve, reject) => {
    baseApi
      .login(loginName, password, code, uuid)
      .then(res => {
        tokenUtil.setAllToken(res.headers[Constant.requestTokenKey], res.headers[Constant.requestRefreshTokenKey]);
        resolve();
      })
      .catch(error => {
        reject(error);
      });
  });
}

// 退出方法
export function logout() {
  return new Promise((resolve, reject) => {
    baseApi
      .logout()
      .then(() => {
        tokenUtil.removeAllToken();
        resolve();
      })
      .catch(error => {
        reject(error);
      });
  });
}

// token失效后退出方法
export function frontLogout() {
  return new Promise((resolve) => {
    tokenUtil.removeAllToken();
    resolve();
  });
}

// 刷新token
export function refreshToken() {
  return new Promise((resolve, reject) => {
    baseApi
      .refreshToken()
      .then((res) => {
        tokenUtil.setAllToken(res.headers[Constant.tokenKey], res.headers[Constant.refreshTokenKey]);
        resolve();
      })
      .catch(error => {
        reject(error);
      });
  });
}
