/**
 * 通用js工具
 */
 import router from '@/router'
 import store from '@/store'
 import * as CryptoUtil from '@/utils/cryptoUtil'
 import {md5} from "@/utils/cryptoUtil";
 const _isEqual = require("lodash/isEqual");
 const _cloneDeep = require("lodash/cloneDeep");
 const _merge = require('lodash/merge')

window.addIframeTab=addIframeTab;
window.addTab=addTab;
window.addEventListener('message', function(e){
         //接收消息，用于在统一用户这边实现iframe打开页面的逻辑
          if(e.data.funName != undefined){
               window[e.data.funName](e.data.param.name,e.data.param.url);
               //addIframeTab(e.data.param.name,e.data.param.url);
          }
          //其他嵌入的其他系统 打开内部路由页面的情况
          if(e.data.msgType != undefined&&e.data.msgType=="openRouterTab"){
               addTab(e.data.data.title,e.data.data.url,e.data.data.param,e.data.data.noCache);
          }
          //嵌入的其他系统 iframe打开第三年方系统的页面
          if(e.data.msgType != undefined&&e.data.msgType=="openIframeTab"){
              addIframeTab(e.data.data.title,e.data.data.url);
          }
  });

/**
 * 打开页签
 * @param {Object} title
 * @param {Object} path 组件路径
 * @param {Object} param
 */
export function addTab(title, path, param,noCache) {
   const vId = CryptoUtil.md5(title+":"+path)
   const rt = path+"/split/"+vId
   let cache = true;
   if(noCache != undefined && noCache != null){
     cache = noCache;
   }
   let route = {
     path: rt,
     component: resolve => require([`@/views${path}`], resolve),
     name: path.substr(1).replace(new RegExp("/", "gm"), "_")+vId,
     noCache:cache,
     hidden: true,
     meta: {
       title: title,
     },
     query:param
   };

   router.addRoute("dynamic_route", route);
   let _r = {
     path: rt
   }
   if(param){
     _r["query"]=param
   }
   router.push(_r);
   store.dispatch("tagsView/addView", route);
 }

 /**
  * 打开页签
  * @param {Object} title
  * @param {Object} path 组件路径
  * @param {Object} param
  */
 export function addIframeTab(title, url,noCache) {
     let path = "/iframe/" + md5(url);
     // 给iFrame组件传参
     let route = {
       path: path,
       component: resolve => require([`@/layout/common/components/iFrame`], resolve),
       name: path.substr(1).replace(new RegExp("/", "gm"), "_"),
       hidden: true,
       props:{src:url},
       meta: {
         title: title,
       }
     };
     router.addRoute("dynamic_route", route);
     let _r = {
       path: path
     }
     router.push(_r);
   }
 /**
  * 关闭页签
  * @param {Object} path
  */
 export function closeTab(path) {
   const visitedViews = store.state.tagsView.visitedViews;
   let view = visitedViews.filter(item => item.path === path)[0];
   if (view) {
     store
       .dispatch("tagsView/delView", view)
       .then(({
         visitedViews
       }) => {
         if (window.location.pathname === path) {
           const latestView = visitedViews.slice(-1)[0];
           if (latestView) {
             router.push(latestView.fullPath).catch(err => err);
           }
         }
       });
   }
 }
 /**
  * 关闭当前页签
  */
 export function closeCurrentTab() {
   const path = window.location.pathname;
   if(path=="/home"){
     return;
   }
   closeTab(path);
 }
 /**
  * 获取主机地址
  * http://host:port
  * @returns
  */
 export function getLocalhostPath() {
   var curWwwPath = window.document.location.href;
   var pathName = window.document.location.pathname;
   var pos = curWwwPath.indexOf(pathName);
   return curWwwPath.substring(0, pos);
 }
 /**
  * 格式化日期
  * @param {*} date
  * @param {*} fmt
  * @returns
  */
 export function formatDate(date, fmt) {
   const $this = new Date(date);
   const o = {
     "M+": $this.getMonth() + 1,
     "d+": $this.getDate(),
     "h+": $this.getHours(),
     "m+": $this.getMinutes(),
     "s+": $this.getSeconds(),
     "q+": Math.floor(($this.getMonth() + 3) / 3),
     S: $this.getMilliseconds()
   };
   if (/(y+)/.test(fmt)) {
     fmt = fmt.replace(
       RegExp.$1,
       ($this.getFullYear() + "").substr(4 - RegExp.$1.length)
     );
   }
   for (const k in o) {
     if (new RegExp("(" + k + ")").test(fmt)) {
       fmt = fmt.replace(
         RegExp.$1,
         RegExp.$1.length === 1 ? o[k] : ("00" + o[k]).substr(("" + o[k]).length)
       );
     }
   }
   return fmt;
 }
 /**
  * 数组删除指定元素(没有返回)
  * @param {*} array
  * @param {*} obj
  */
 export function arrayDeleteObject(array, obj) {
   const index = array.indexOf(obj);
   if (index > -1) {
     array.splice(index, 1);
   }
 }
 /**
  * 合并对象
  * @param {*} target
  * @param {*} source
  * @returns
  */
 export function merge(target, source) {
   return _merge(target, source);
 }
 /**
  * 深拷贝
  * @param {*} source  要深拷贝的值
  * @return {*} 返回拷贝后的值
  */
 export function cloneDeep(source) {
   return _cloneDeep(source);
 }
 /**
  * 判断两个对象是否相等
  * @param {*} object 对象1
  * @param {*} other 对象2
  * @return {boolean}
  */
 export function isEqual(object, other) {
   return _isEqual(object, other);
 }
 /**
  * 是不是字符串
  * @param {*} str
  * @returns
  */
 export function isString(str) {
   if (typeof str === "string" || str instanceof String) {
     return true;
   }
   return false;
 }
 /**
  * 是不是数组
  * @param {*} arg
  * @returns
  */
 export function isArray(arg) {
   if (typeof Array.isArray === "undefined") {
     return Object.prototype.toString.call(arg) === "[object Array]";
   }
   return Array.isArray(arg);
 }
 /**
  * 是不是对象
  * @param {*} arg
  * @returns
  */
 export function isObject(arg) {
   return Object.prototype.toString.call(arg) === "[object Object]";
 }
 /**
  * 是不是方法
  * @param {*} arg
  * @returns
  */
 export function isFunction(arg) {
   return typeof arg == "function";
 }
 /**
  * 是不是外部地址
  * @param {*} path
  * @returns
  */
 export function isExternal(path) {
   return /^(https?:|mailto:|tel:)/.test(path);
 }
 /**
  * 打开windows窗口
  * @param {Sting} url
  * @param {Sting} title
  * @param {Number} w
  * @param {Number} h
  */
 export function openWindow(url, title, w, h) {
   // Fixes dual-screen position                            Most browsers       Firefox
   const dualScreenLeft =
     window.screenLeft !== undefined ? window.screenLeft : screen.left;
   const dualScreenTop =
     window.screenTop !== undefined ? window.screenTop : screen.top;

  const width = window.innerWidth ?
    window.innerWidth :
    document.documentElement.clientWidth ?
    document.documentElement.clientWidth :
    screen.width;
  const height = window.innerHeight ?
    window.innerHeight :
    document.documentElement.clientHeight ?
    document.documentElement.clientHeight :
    screen.height;

  const left = width / 2 - w / 2 + dualScreenLeft;
  const top = height / 2 - h / 2 + dualScreenTop;
  const newWindow = window.open(
    url,
    title,
    "toolbar=no, location=no, directories=no, status=no, menubar=no, scrollbars=no, resizable=yes, copyhistory=no, width=" +
    w +
    ", height=" +
    h +
    ", top=" +
    top +
    ", left=" +
    left
  );

  // Puts focus on the newWindow
  if (window.focus) {
    newWindow.focus();
  }
}
/**
 * 设置px像素
 * @param {*} val
 * @param {*} defval
 * @returns
 */
export function setPx(val, defval = "") {
  if (!val) val = defval;
  if (!val) return "";
  val = val + "";
  if (val.indexOf("px") === -1) {
    if (val.indexOf("%") === -1) {
      val = val + "px";
    }
  }
  return val;
}
/**
 * 随机ID
 * @returns
 */
export function randomId() {
  let $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
  let maxPos = $chars.length;
  let id = '';
  for (let i = 0; i < 16; i++) {
    id += $chars.charAt(Math.floor(Math.random() * maxPos));
  }
  return id;
}
/**
 * 获取Link对象--用作对url的处理,以下是对象的方法
 * getUrl()-获取完整url
 * go()-执行跳转
 * set(key,value)-设置参数
 * get(key)-获取参数
 * getAll()-获取所有参数
 * remove(key)-删除参数
 * @param {*} url 为空时默认当前地址
 * @returns
 */
export function getLink(url) {
  if (!url) {
    url = window.location.href;
  }
  // 解析
  var _parms = {};
  var _path = url.split("?")[0];
  var _query = url.split("?")[1];
  if (_query) {
    var _queryArr = _query.split("&");
    _queryArr.forEach(function(item) {
      var _value = item.split("=")[1];
      var _key = item.split("=")[0];
      _parms[_key] = _value;
    });
  }
  //对象
  var _link = {
    path: _path,
    parms: _parms,
    //获取完整地址
    getUrl: function() {
      var _p = "";
      var _isFrist = true;
      for (var pk in this.parms) {
        if (_isFrist) {
          _p += "?" + pk + "=" + this.parms[pk];
          _isFrist = false;
        } else {
          _p += "&" + pk + "=" + this.parms[pk];
        }
      }
      return this.path + _p;
    },
    //跳转
    go: function() {
      window.location.href = this.getUrl();
    },
    //设置参数
    set: function(key, value) {
      if (typeof key == "object") {
        for (var i in key) {
          if (typeof i == "function") continue;
          this.parms[i] = key[i];
        }
      } else {
        this.parms[key] =
          typeof value == "undefined" ? $("#" + key).val() : value;
      }
      return this;
    },
    //取参数
    get: function(key) {
      return this.parms[key];
    },
    //取所有参数
    getAll: function() {
      return this.parms;
    },
    //删除参数
    remove: function(key) {
      return delete this.parms[key];
    }
  };
  return _link;
}

/**
 * Check if an element has a class
 * @param {HTMLElement} elm
 * @param {string} cls
 * @returns {boolean}
 */
export function hasClass(ele, cls) {
  return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'))
}

/**
 * Add class to element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function addClass(ele, cls) {
  if (!hasClass(ele, cls)) ele.className += ' ' + cls
}

/**
 * Remove class from element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function removeClass(ele, cls) {
  if (hasClass(ele, cls)) {
    const reg = new RegExp('(\\s|^)' + cls + '(\\s|$)')
    ele.className = ele.className.replace(reg, ' ')
  }
}
