import CryptoJS from "crypto-js";

// 预置公钥
const publicKey =
  "MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBAKab6y3+C+uBW9iQr1Cf5sZDD5VDsmP+" +
  "DiAH6UedxT/nkQu3g0pIoLmvI8SP0fnbYDtOskMkKA2Iq2sFthDZd8cCAwEAAQ==";
// 预置私钥
const privateKey =
  "MIIBVQIBADANBgkqhkiG9w0BAQEFAASCAT8wggE7AgEAAkEAppvrLf4L64Fb2JCv" +
  "UJ/mxkMPlUOyY/4OIAfpR53FP+eRC7eDSkigua8jxI/R+dtgO06yQyQoDYirawW2" +
  "ENl3xwIDAQABAkBZz7L3F/tU+GaKV3QqAghk18GOtO90936lxZ36/RFXYJ3vUwt/" +
  "2qiXNiPl0gxlXtCi0hNUNkLtJ9vEtDo1VdABAiEA1+AzO1AgE3HzN2/nHkiLM97W" +
  "QssGrBBAYcZoHv0IJhUCIQDFk4J2rkmmbROcXCbyXQ7aueZre0rEFJ/gC3TJU7OZ" +
  "awIhALQpqbgbo9l2pTwkCSft9TcCB6FQ3a2mFoVDV8xc8nsNAiA7YY83fkByfZwE" +
  "RYoLE77wLCk9qouZDR3T532NONPkywIhAJigfIDXzwXa/kx09ywQQKPj4DWdGHMQ" +
  "Z/p51evKXBYO";
// 签名公钥
const signKey =
  "MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAqLISwAUln0JjEK+Jeld/" +
  "k6HoiOfak1aWJphbcJnQvKxfbKXxq53M43Z/eKHTWFxo6whON69Fun0uy5Izqm0b" +
  "ibKBCcHofSrEsjI9OPXpviQSVYmoNsjdDcHkcx8ZWG0bWc9qiivqd1OZg2H09c1N" +
  "R7hourCE6YO/h00iTpCBSn9WBLxz2jEZuRTaa7PM2cw2TEsnkeELiPZlfuXY1rhC" +
  "WcHP0VFV1I+hAesLtqyL5x1Tk9/gVpKdSR4JRZcbvc9fDlsDsFd/HX+rNsOvKc22" +
  "MSuGd3tQJIx9aSpP8/qNLlHAz8AtKi3mqw0LKCWq2Z2YHWz32aaBBeBwkRNXIryO" +
  "hQIDAQAB";
/**
 * 使用预置的公钥加密
 * @param {*} text
 * @returns
 */
export function encrypt(text) {
  return encryptByDES(text, publicKey);
}

/**
 * 使用预置的私钥解密
 * @param {*} text
 * @returns
 */
export function decrypt(text) {
  return decryptByDES(text, privateKey);
}

/**
 * 加密
 * @param {string} message 需要加密的字符串
 * @param {string} key 秘钥
 * @return {string}
 */
export function encryptByDES(message, key) {
  const keyHex = CryptoJS.enc.Utf8.parse(key);
  const encrypted = CryptoJS.DES.encrypt(message, keyHex, {
    mode: CryptoJS.mode.ECB,
    padding: CryptoJS.pad.Pkcs7
  });
  return encrypted.ciphertext.toString();
}

/**
 * 解密
 * @param {string} cipherText 密文
 * @param {string} key 秘钥
 * @return {string}
 */
export function decryptByDES(cipherText, key) {
  const keyHex = CryptoJS.enc.Utf8.parse(key);
  const decrypted = CryptoJS.DES.decrypt(
    {
      ciphertext: CryptoJS.enc.Hex.parse(cipherText)
    },
    keyHex,
    {
      mode: CryptoJS.mode.ECB,
      padding: CryptoJS.pad.Pkcs7
    }
  );
  return decrypted.toString(CryptoJS.enc.Utf8);
}

/**
 * MD5 加密
 * @param {String} text 原始文本
 * @returns {*} 转md5否的文本
 */
export function md5(text) {
  return CryptoJS.MD5(text).toString();
}

/**
 * Base64转码
 * @param {String} text 文本
 * @returns {string} base64文本
 */
export function base64Encode(text) {
  const wordArray = CryptoJS.enc.Utf8.parse(text);
  return CryptoJS.enc.Base64.stringify(wordArray);
}

/**
 * Base64 解码
 * @param {String} text base64密文
 * @returns {string} 原始文本
 */
export function base64Decode(text) {
  const parsedWordArray = CryptoJS.enc.Base64.parse(text);
  return parsedWordArray.toString(CryptoJS.enc.Utf8);
}

// 生成uuid
export function uuid() {
  const s = [];
  const hexDigits = "0123456789abcdef";
  for (let i = 0; i < 36; i++) {
    s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
  }
  s[14] = "4";
  s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1);
  s[8] = s[13] = s[18] = s[23] = "-";
  return s.join("");
}
// 创建签名（需要与后端共同使用）
export function createSign(timestamp, nonce, requestParameters) {
  const signEncrypt = function(string, iterations) {
    // 签名最大长度（多了的截取，保留前后，去除中间）
    const signMaxLength = 256;
    let result = string;
    if (iterations) {
      for (let i = 0; i < iterations; i++) {
        result = _md5(result);
      }
    } else {
      result = _md5(result);
    }
    // 需要对结果进行截取判断
    if (result.length > signMaxLength) {
      const front = result.substring(0, signMaxLength / 2);
      const behind = result.substring(
        result.length - signMaxLength / 2,
        result.length
      );
      return front + "$" + behind;
    }
    return result;
  };
  const _md5 = function(string) {
    let x;
    let k, AA, BB, CC, DD, a, b, c, d;
    const S11 = 7,
      S12 = 12,
      S13 = 17,
      S14 = 22;
    const S21 = 5,
      S22 = 9,
      S23 = 14,
      S24 = 20;
    const S31 = 4,
      S32 = 11,
      S33 = 16,
      S34 = 23;
    const S41 = 6,
      S42 = 10,
      S43 = 15,
      S44 = 21;
    string = uTF8Encode(string);
    x = convertToWordArray(string);
    a = 0x67452301;
    b = 0xefcdab89;
    c = 0x98badcfe;
    d = 0x10325476;
    for (k = 0; k < x.length; k += 16) {
      AA = a;
      BB = b;
      CC = c;
      DD = d;
      a = FF(a, b, c, d, x[k + 0], S11, 0xd76aa478);
      d = FF(d, a, b, c, x[k + 1], S12, 0xe8c7b756);
      c = FF(c, d, a, b, x[k + 2], S13, 0x242070db);
      b = FF(b, c, d, a, x[k + 3], S14, 0xc1bdceee);
      a = FF(a, b, c, d, x[k + 4], S11, 0xf57c0faf);
      d = FF(d, a, b, c, x[k + 5], S12, 0x4787c62a);
      c = FF(c, d, a, b, x[k + 6], S13, 0xa8304613);
      b = FF(b, c, d, a, x[k + 7], S14, 0xfd469501);
      a = FF(a, b, c, d, x[k + 8], S11, 0x698098d8);
      d = FF(d, a, b, c, x[k + 9], S12, 0x8b44f7af);
      c = FF(c, d, a, b, x[k + 10], S13, 0xffff5bb1);
      b = FF(b, c, d, a, x[k + 11], S14, 0x895cd7be);
      a = FF(a, b, c, d, x[k + 12], S11, 0x6b901122);
      d = FF(d, a, b, c, x[k + 13], S12, 0xfd987193);
      c = FF(c, d, a, b, x[k + 14], S13, 0xa679438e);
      b = FF(b, c, d, a, x[k + 15], S14, 0x49b40821);
      a = GG(a, b, c, d, x[k + 1], S21, 0xf61e2562);
      d = GG(d, a, b, c, x[k + 6], S22, 0xc040b340);
      c = GG(c, d, a, b, x[k + 11], S23, 0x265e5a51);
      b = GG(b, c, d, a, x[k + 0], S24, 0xe9b6c7aa);
      a = GG(a, b, c, d, x[k + 5], S21, 0xd62f105d);
      d = GG(d, a, b, c, x[k + 10], S22, 0x2441453);
      c = GG(c, d, a, b, x[k + 15], S23, 0xd8a1e681);
      b = GG(b, c, d, a, x[k + 4], S24, 0xe7d3fbc8);
      a = GG(a, b, c, d, x[k + 9], S21, 0x21e1cde6);
      d = GG(d, a, b, c, x[k + 14], S22, 0xc33707d6);
      c = GG(c, d, a, b, x[k + 3], S23, 0xf4d50d87);
      b = GG(b, c, d, a, x[k + 8], S24, 0x455a14ed);
      a = GG(a, b, c, d, x[k + 13], S21, 0xa9e3e905);
      d = GG(d, a, b, c, x[k + 2], S22, 0xfcefa3f8);
      c = GG(c, d, a, b, x[k + 7], S23, 0x676f02d9);
      b = GG(b, c, d, a, x[k + 12], S24, 0x8d2a4c8a);
      a = HH(a, b, c, d, x[k + 5], S31, 0xfffa3942);
      d = HH(d, a, b, c, x[k + 8], S32, 0x8771f681);
      c = HH(c, d, a, b, x[k + 11], S33, 0x6d9d6122);
      b = HH(b, c, d, a, x[k + 14], S34, 0xfde5380c);
      a = HH(a, b, c, d, x[k + 1], S31, 0xa4beea44);
      d = HH(d, a, b, c, x[k + 4], S32, 0x4bdecfa9);
      c = HH(c, d, a, b, x[k + 7], S33, 0xf6bb4b60);
      b = HH(b, c, d, a, x[k + 10], S34, 0xbebfbc70);
      a = HH(a, b, c, d, x[k + 13], S31, 0x289b7ec6);
      d = HH(d, a, b, c, x[k + 0], S32, 0xeaa127fa);
      c = HH(c, d, a, b, x[k + 3], S33, 0xd4ef3085);
      b = HH(b, c, d, a, x[k + 6], S34, 0x4881d05);
      a = HH(a, b, c, d, x[k + 9], S31, 0xd9d4d039);
      d = HH(d, a, b, c, x[k + 12], S32, 0xe6db99e5);
      c = HH(c, d, a, b, x[k + 15], S33, 0x1fa27cf8);
      b = HH(b, c, d, a, x[k + 2], S34, 0xc4ac5665);
      a = II(a, b, c, d, x[k + 0], S41, 0xf4292244);
      d = II(d, a, b, c, x[k + 7], S42, 0x432aff97);
      c = II(c, d, a, b, x[k + 14], S43, 0xab9423a7);
      b = II(b, c, d, a, x[k + 5], S44, 0xfc93a039);
      a = II(a, b, c, d, x[k + 12], S41, 0x655b59c3);
      d = II(d, a, b, c, x[k + 3], S42, 0x8f0ccc92);
      c = II(c, d, a, b, x[k + 10], S43, 0xffeff47d);
      b = II(b, c, d, a, x[k + 1], S44, 0x85845dd1);
      a = II(a, b, c, d, x[k + 8], S41, 0x6fa87e4f);
      d = II(d, a, b, c, x[k + 15], S42, 0xfe2ce6e0);
      c = II(c, d, a, b, x[k + 6], S43, 0xa3014314);
      b = II(b, c, d, a, x[k + 13], S44, 0x4e0811a1);
      a = II(a, b, c, d, x[k + 4], S41, 0xf7537e82);
      d = II(d, a, b, c, x[k + 11], S42, 0xbd3af235);
      c = II(c, d, a, b, x[k + 2], S43, 0x2ad7d2bb);
      b = II(b, c, d, a, x[k + 9], S44, 0xeb86d391);
      a = addUnsigned(a, AA);
      b = addUnsigned(b, BB);
      c = addUnsigned(c, CC);
      d = addUnsigned(d, DD);
    }
    const tempValue = wordToHex(a) + wordToHex(b) + wordToHex(c) + wordToHex(d);
    return tempValue.toLowerCase();
  };
  const rotateLeft = function(lValue, iShiftBits) {
    return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
  };
  const addUnsigned = function(lX, lY) {
    let lX4, lY4, lX8, lY8, lResult;
    lX8 = lX & 0x80000000;
    lY8 = lY & 0x80000000;
    lX4 = lX & 0x40000000;
    lY4 = lY & 0x40000000;
    lResult = (lX & 0x3fffffff) + (lY & 0x3fffffff);
    if (lX4 & lY4) return lResult ^ 0x80000000 ^ lX8 ^ lY8;
    if (lX4 | lY4) {
      if (lResult & 0x40000000) return lResult ^ 0xc0000000 ^ lX8 ^ lY8;
      else return lResult ^ 0x40000000 ^ lX8 ^ lY8;
    } else {
      return lResult ^ lX8 ^ lY8;
    }
  };
  const F = function(x, y, z) {
    return (x & y) | (~x & z);
  };
  const G = function(x, y, z) {
    return (x & z) | (y & ~z);
  };
  const H = function(x, y, z) {
    return x ^ y ^ z;
  };
  const I = function(x, y, z) {
    return y ^ (x | ~z);
  };
  const FF = function(a, b, c, d, x, s, ac) {
    a = addUnsigned(a, addUnsigned(addUnsigned(F(b, c, d), x), ac));
    return addUnsigned(rotateLeft(a, s), b);
  };
  const GG = function(a, b, c, d, x, s, ac) {
    a = addUnsigned(a, addUnsigned(addUnsigned(G(b, c, d), x), ac));
    return addUnsigned(rotateLeft(a, s), b);
  };
  const HH = function(a, b, c, d, x, s, ac) {
    a = addUnsigned(a, addUnsigned(addUnsigned(H(b, c, d), x), ac));
    return addUnsigned(rotateLeft(a, s), b);
  };
  const II = function(a, b, c, d, x, s, ac) {
    a = addUnsigned(a, addUnsigned(addUnsigned(I(b, c, d), x), ac));
    return addUnsigned(rotateLeft(a, s), b);
  };
  const convertToWordArray = function(string) {
    let lWordCount;
    const lMessageLength = string.length;
    const lNumberOfWordsTempOne = lMessageLength + 8;
    const lNumberOfWordsTempTwo =
      (lNumberOfWordsTempOne - (lNumberOfWordsTempOne % 64)) / 64;
    const lNumberOfWords = (lNumberOfWordsTempTwo + 1) * 16;
    const lWordArray = Array(lNumberOfWords - 1);
    let lBytePosition = 0;
    let lByteCount = 0;
    while (lByteCount < lMessageLength) {
      lWordCount = (lByteCount - (lByteCount % 4)) / 4;
      lBytePosition = (lByteCount % 4) * 8;
      lWordArray[lWordCount] =
        lWordArray[lWordCount] |
        (string.charCodeAt(lByteCount) << lBytePosition);
      lByteCount++;
    }
    lWordCount = (lByteCount - (lByteCount % 4)) / 4;
    lBytePosition = (lByteCount % 4) * 8;
    lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
    lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
    lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
    return lWordArray;
  };
  const wordToHex = function(lValue) {
    let WordToHexValue = "",
      WordToHexValueTemp = "",
      lByte,
      lCount;
    for (lCount = 0; lCount <= 3; lCount++) {
      lByte = (lValue >>> (lCount * 8)) & 255;
      WordToHexValueTemp = "0" + lByte.toString(16);
      WordToHexValue =
        WordToHexValue +
        WordToHexValueTemp.substr(WordToHexValueTemp.length - 2, 2);
    }
    return WordToHexValue;
  };
  const uTF8Encode = function(string) {
    string = string.toString().replace(/\x0d\x0a/g, "\x0a");
    let output = "";
    for (let n = 0; n < string.length; n++) {
      const c = string.charCodeAt(n);
      if (c < 128) {
        output += String.fromCharCode(c);
      } else if (c > 127 && c < 2048) {
        output += String.fromCharCode((c >> 6) | 192);
        output += String.fromCharCode((c & 63) | 128);
      } else {
        output += String.fromCharCode((c >> 12) | 224);
        output += String.fromCharCode(((c >> 6) & 63) | 128);
        output += String.fromCharCode((c & 63) | 128);
      }
    }
    return output;
  };
  const sortDesc = function(array) {
    for (let i = 0; i < array.length; i++) {
      const b = i + 1;
      if (array[i] < array[b]) {
        let j = b - 1,
          ayb = array[b];
        while (ayb > array[j]) {
          array[j + 1] = array[j];
          j--;
        }
        array[j + 1] = ayb;
      }
    }
  };

  const sortAsc = function(array) {
    for (let i = 0; i < array.length; i++) {
      const b = i + 1;
      if (array[i] > array[b]) {
        let j = b - 1,
          ayb = array[b];
        while (ayb < array[j]) {
          array[j + 1] = array[j];
          j--;
        }

        array[j + 1] = ayb;
      }
    }
  };
  const generateSign = function(_requestParameters) {
    const keys = [];
    for (const key in _requestParameters) {
      keys.push(key);
    }
    sortDesc(keys);
    const values = [];
    for (let i = 0; i < keys.length; i++) {
      if (i % 2 == 0) {
        values.push(_requestParameters[keys[i]]);
      }
    }
    let toSign = "";
    for (let i = 0; i < values.length; i++) {
      if (values[i] instanceof Array) {
        sortAsc(values[i]);
        for (let v = 0; v < values[i].length; v++) {
          toSign += encodeURIComponent(values[i][v]);
        }
      } else {
        toSign += encodeURIComponent(values[i]);
      }
    }
    const reg1 = new RegExp("!", "g");
    toSign = toSign.replace(reg1, "%21");
    const reg2 = new RegExp("\\)", "g");
    toSign = toSign.replace(reg2, "%29");
    const reg3 = new RegExp("\\(", "g");
    toSign = toSign.replace(reg3, "%28");
    const reg4 = new RegExp("~", "g");
    toSign = toSign.replace(reg4, "%7E");
    toSign += timestamp + nonce;
    return signEncrypt(toSign, 5);
  };
  return generateSign(requestParameters);
}
