import { Notification, MessageBox, Message } from "element-ui";
import { isObject } from "./commonUtil";

// 成功提示信息
export function success(msg) {
  Message.success(msg);
}
// 警告提示信息
export function warning(msg) {
  Message.warning(msg);
}
// 普通提示信息
export function info(msg) {
  Message.info(msg);
}
// 失败提示信息
export function error(msg) {
  Message.error(msg);
}
// 弹出框
export function alert(msg, callback, confirmButtonText, title) {
  if (isObject(callback)) {
    MessageBox.alert(msg, callback);
  }
  if (!title) {
    title = "提示";
  }
  if (!confirmButtonText) {
    confirmButtonText = "确定";
  }
  MessageBox.alert(msg, title, {
    confirmButtonText: confirmButtonText,
    callback: action => {
      if (callback) {
        callback(action);
      }
    }
  }).then(r => {
    console.log(r);
  });
}
// 成功提示信息
export function alertSuccess(msg) {
  MessageBox.alert(msg, "提示", {
    type: "success"
  }).then(r => {
    console.log(r);
  });
}
// 警告提示信息
export function alertWarning(msg) {
  MessageBox.alert(msg, "提示", {
    type: "warning"
  }).then(r => {
    console.log(r);
  });
}
// 普通提示信息
export function alertInfo(msg) {
  MessageBox.info(msg, "提示", {
    type: "success"
  }).then(r => {
    console.log(r);
  });
}
// 失败提示信息
export function alertError(msg) {
  MessageBox.alert(msg, "提示", {
    type: "error"
  }).then(r => {
    console.log(r);
  });
}
// 弹出框（弹出内容为html）
export function alertHtml(msg, callback, confirmButtonText, title) {
  if (!title) {
    title = "提示";
  }
  if (!confirmButtonText) {
    confirmButtonText = "确定";
  }
  MessageBox.alert(msg, title, {
    dangerouslyUseHTMLString: true,
    confirmButtonText: confirmButtonText,
    callback: action => {
      if (callback) {
        callback(action);
      }
    }
  }).then(r => {
    console.log(r);
  });
}
// 确定框
export function confirm(
  msg,
  successCallback,
  errorCallback,
  type,
  title,
  confirmButtonText,
  cancelButtonText,
  iconClass
) {
  if (!title && title != "") {
    title = "警告";
  }
  if (!confirmButtonText && confirmButtonText != "") {
    confirmButtonText = "确定";
  }
  if (!cancelButtonText && cancelButtonText != "") {
    cancelButtonText = "取消";
  }
  if (!type) {
    type = "warning";
  }
  MessageBox.confirm(msg, title, {
    confirmButtonText: confirmButtonText,
    cancelButtonText: cancelButtonText,
    type: type,
	iconClass:iconClass
  })
    .then(function() {
      if (successCallback) {
        successCallback();
      }
    })
    .catch(() => {
      if (errorCallback) {
        errorCallback();
      }
    });
}
// 填写框
export function prompt(
  msg,
  successCallback,
  errorCallback,
  inputPattern,
  inputErrorMessage,
  title,
  confirmButtonText,
  cancelButtonText
) {
  if (!title) {
    title = "提示";
  }
  if (!confirmButtonText) {
    confirmButtonText = "确定";
  }
  if (!cancelButtonText) {
    cancelButtonText = "取消";
  }
  if (!inputPattern) {
    inputPattern = "";
  }
  if (!inputErrorMessage) {
    inputErrorMessage = "";
  }
  MessageBox.prompt(msg, title, {
    confirmButtonText: confirmButtonText,
    cancelButtonText: cancelButtonText,
    inputPattern: inputPattern,
    inputErrorMessage: inputErrorMessage
  })
    .then(({ value }) => {
      if (successCallback) {
        successCallback(value);
      }
    })
    .catch(() => {
      if (errorCallback) {
        errorCallback();
      }
    });
}
// 通知
export function notify(msg, onClick, title) {
  if (!title) {
    title = "提示";
  }
  Notification({
    title: title,
    message: msg,
    onClick: action => {
      if (onClick) {
        onClick(action);
      }
    }
  }).then(r => {
    console.log(r);
  });
}
// 成功提示信息
export function notifySuccess(msg) {
  Notification.success(msg);
}
// 警告提示信息
export function notifyWarning(msg) {
  Notification.warning(msg);
}
// 普通提示信息
export function notifyInfo(msg) {
  Notification.info(msg);
}
// 失败提示信息
export function notifyError(msg) {
  Notification.error(msg);
}
// 通知（弹出内容为html）
export function notifyHtml(msg, onClick, title) {
  if (!title) {
    title = "提示";
  }
  Notification({
    title: title,
    message: msg,
    dangerouslyUseHTMLString: true,
    onClick: action => {
      if (onClick) {
        onClick(action);
      }
    }
  }).then(r => {
    console.log(r);
  });
}
