import Cookies from "js-cookie";

/**
 * 以下是缓存处理
 * local:只能手动删除，否则一直有效（高版本浏览器才支持，IE8及以上，存储过多页面会很卡）
 * session:针对标签页，关闭标签页就算不关闭浏览器也作废了（高版本浏览器才支持，IE8及以上，存储过多页面会很卡）
 * cookie:针对浏览器，关闭浏览器就作废了（浏览器几乎通用，还可以指定有效期）
 */
// 设置永久缓存(json对象)
export function setLocalObject(key, val) {
  window.localStorage.setItem(key, JSON.stringify(val));
}
// 获取永久缓存(json对象)
export function getLocalObject(key) {
  let json = window.localStorage.getItem(key);
  return JSON.parse(json);
}
// 设置永久缓存(字符串)
export function setLocalString(key, val) {
  window.localStorage.setItem(key, val);
}
// 获取永久缓存(字符串)
export function getLocalString(key) {
  return window.localStorage.getItem(key);
}
// 移除永久缓存
export function removeLocal(key) {
  window.localStorage.removeItem(key);
}
// 清除全部永久缓存
export function clearLocal() {
  window.localStorage.clear();
}
// 设置临时缓存(对象)
export function setSessionObject(key, val) {
  window.sessionStorage.setItem(key, JSON.stringify(val));
}
// 获取临时缓存(对象)
export function getSessionObject(key) {
  let json = window.sessionStorage.getItem(key);
  return JSON.parse(json);
}
// 设置临时缓存(字符串)
export function setSessionString(key, val) {
  window.sessionStorage.setItem(key, val);
}
// 获取临时缓存(字符串)
export function getSessionString(key) {
  return window.sessionStorage.getItem(key);
}
// 移除临时缓存
export function removeSession(key) {
  window.sessionStorage.removeItem(key);
}
// 清空全部临时缓存
export function clearSession() {
  window.sessionStorage.clear();
}
// 设置临时缓存
export function setCookie(key, data) {
  Cookies.set(key, data);
}
// 设置临时缓存(自定义参数)
export function setCookieParams(key, data, params) {
  Cookies.set(key, data, params);
}
// 获取临时缓存
export function getCookie(key) {
  return Cookies.get(key);
}
// 删除临时缓存
export function removeCookie(key) {
  Cookies.remove(key);
}
// 清空临时缓存
export function clearCookie() {
  // 获取所有cookie
  const all = Cookies.get();
  for (const key in all) {
    // 依次清空cookie
    Cookies.remove(key);
  }
}
// 清空所有缓存
export function clearAll() {
  clearLocal();
  clearCookie();
  clearCookie();
}
